(**************************************************************************)
(*                                                                        *)
(*  Setup for SFtpServer                                                  *)
(*  Copyright (C) 2022   Peter Moylan                                     *)
(*                                                                        *)
(*  This program is free software: you can redistribute it and/or modify  *)
(*  it under the terms of the GNU General Public License as published by  *)
(*  the Free Software Foundation, either version 3 of the License, or     *)
(*  (at your option) any later version.                                   *)
(*                                                                        *)
(*  This program is distributed in the hope that it will be useful,       *)
(*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *)
(*  GNU General Public License for more details.                          *)
(*                                                                        *)
(*  You should have received a copy of the GNU General Public License     *)
(*  along with this program.  If not, see <http://www.gnu.org/licenses/>. *)
(*                                                                        *)
(*  To contact author:   http://www.pmoylan.org   peter@pmoylan.org       *)
(*                                                                        *)
(**************************************************************************)

IMPLEMENTATION MODULE Clipboard;

        (****************************************************************)
        (*                                                              *)
        (*                  PM Setup for SFtpServer                     *)
        (*            Clipboard for copy/paste operations               *)
        (*                                                              *)
        (*        Last edited:    26 September 2022                     *)
        (*        Status:         OK                                    *)
        (*                                                              *)
        (****************************************************************)


FROM IPFilters IMPORT
    (* type *)  ListPtr;

FROM Storage IMPORT
    (* proc *)  ALLOCATE, DEALLOCATE;

(**************************************************************************)

VAR SavedList: ListPtr;

(**************************************************************************)

PROCEDURE DiscardList (VAR (*INOUT*) list: ListPtr);

    (* Destroys the list. *)

    VAR current: ListPtr;

    BEGIN
        WHILE list <> NIL DO
            current := list;
            list := current^.next;
            DISPOSE (current);
        END (*WHILE*);
    END DiscardList;

(**************************************************************************)

PROCEDURE CopyOf (list: ListPtr): ListPtr;

    (* Returns a copy of the list. *)

    VAR result, previous, current: ListPtr;

    BEGIN
        result := NIL;  previous := NIL;
        WHILE list <> NIL DO
            NEW (current);
            current^ := list^;
            current^.previous := previous;
            IF previous = NIL THEN
                result := current;
            ELSE
                previous^.next := current;
            END (*IF*);
            current^.next := NIL;
            previous := current;
            list := list^.next;
        END (*WHILE*);
        RETURN result;
    END CopyOf;

(**************************************************************************)

PROCEDURE SaveCopy (list: ListPtr);

    (* Saves this list, discarding any previously saved list. *)

    BEGIN
        IF list <> NIL THEN
            DiscardList (SavedList);
            SavedList := list;
        END (*IF*);
    END SaveCopy;

(**************************************************************************)

PROCEDURE RetrieveCopy(): ListPtr;

    (* Returns a copy of the last saved list. *)

    BEGIN
        RETURN CopyOf(SavedList);
    END RetrieveCopy;

(**************************************************************************)

BEGIN
    SavedList := NIL;
FINALLY
    DiscardList (SavedList);
END Clipboard.

