(**************************************************************************)
(*                                                                        *)
(*  Setup for SFtpServer                                                  *)
(*  Copyright (C) 2022   Peter Moylan                                     *)
(*                                                                        *)
(*  This program is free software: you can redistribute it and/or modify  *)
(*  it under the terms of the GNU General Public License as published by  *)
(*  the Free Software Foundation, either version 3 of the License, or     *)
(*  (at your option) any later version.                                   *)
(*                                                                        *)
(*  This program is distributed in the hope that it will be useful,       *)
(*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *)
(*  GNU General Public License for more details.                          *)
(*                                                                        *)
(*  You should have received a copy of the GNU General Public License     *)
(*  along with this program.  If not, see <http://www.gnu.org/licenses/>. *)
(*                                                                        *)
(*  To contact author:   http://www.pmoylan.org   peter@pmoylan.org       *)
(*                                                                        *)
(**************************************************************************)

IMPLEMENTATION MODULE LogPage;

        (****************************************************************)
        (*                                                              *)
        (*                    PM Setup for SFtpServer                   *)
        (*                 Logging page of the notebook                 *)
        (*                                                              *)
        (*        Last edited:    27 July 2022                          *)
        (*        Status:         OK                                    *)
        (*                                                              *)
        (****************************************************************)


FROM SYSTEM IMPORT ADDRESS, INT16, CAST, ADR;

IMPORT OS2, OS2RTL, DID, Strings, CommonSettings;

FROM Languages IMPORT
    (* type *)  LangHandle,
    (* proc *)  StrToBuffer;

FROM FSUINI IMPORT
    (* type *)  FHINI,
    (* proc *)  OpenINIFiles, CloseINIFiles,
                INIFetch, INIGetCard, INIGetString, INIPut, INIPutString;

FROM MiscFuncs IMPORT
    (* type *)  CharArrayPointer,
    (* proc *)  EVAL;

FROM Misc IMPORT
    (* proc *)  WinSetDlgItemCard, WinQueryDlgItemCard;

FROM Names IMPORT
    (* type *)  UserName, UserNameIndex, FilenameString;

FROM LowLevel IMPORT
    (* proc *)  IAND, IOR;

FROM Storage IMPORT
    (* proc *)  ALLOCATE, DEALLOCATE;

(**************************************************************************)

CONST Nul = CHR(0);

TYPE
    TwoCard = ARRAY [0..1] OF CARDINAL;
    DirectoryString = ARRAY [0..511] OF CHAR;
    TextArray = ARRAY [0..63] OF CHAR;
    TextPointer = POINTER TO TextArray;

VAR
    ChangeInProgress: BOOLEAN;
    FTP, SFTP: BOOLEAN;
    PageID: CARDINAL;
    pagehandle, notebookhandle: OS2.HWND;

    (* Original values of the ini variables. *)

    OldLogLevel, OldTransLevel, OldTransLevelS: CARDINAL;
    OldUserLogName, OldCommonLogName,
                     OldTransLogName, OldTransLogNameS, OldSyslogHost: FilenameString;
    OldLogSITEMNGR: BOOLEAN;

    (* Text for a spinbutton control. *)

    LogDetailText: ARRAY [0..3] OF TextArray;
    LogDetailPtr: ARRAY [0..3] OF TextPointer;

(************************************************************************)
(*                    OPERATIONS ON DIALOGUE LABELS                     *)
(************************************************************************)

PROCEDURE SetLanguage (lang: LangHandle);

    (* Relabels this page in the new language. *)

    VAR stringval: ARRAY [0..511] OF CHAR;

    (********************************************************************)

    PROCEDURE SetOne (item: ARRAY OF CHAR;  place: CARDINAL);

        BEGIN
            StrToBuffer (lang, item, stringval);
            OS2.WinSetDlgItemText (pagehandle, place, stringval);
        END SetOne;

    (********************************************************************)

    BEGIN
        StrToBuffer (lang, "Logging.tab", stringval);
        OS2.WinSendMsg (notebookhandle, OS2.BKM_SETTABTEXT,
                        CAST(ADDRESS,PageID), ADR(stringval));
        SetOne ("Logging.Transfer", DID.TransferLogBox);
        SetOne ("Logging.UserLog", DID.UserLog);
        SetOne ("Logging.CommonLog", DID.CommonLog);

        SetOne ("Logging.FTPTransaction", DID.TransactionLogBox);
        SetOne ("Logging.DetailedLogTo", DID.DetailedLogTo);
        SetOne ("Logging.Disk", DID.DiskLog);
        SetOne ("Logging.Screen", DID.ScreenLog);
        SetOne ("Logging.Pipe", DID.PipeLog);
        SetOne ("Logging.Syslog", DID.SysLog);
        SetOne ("Logging.TransLogName", DID.LogFileName);

        SetOne ("Logging.SFTPTransaction", DID.TransactionLogBoxS);
        SetOne ("Logging.DetailedLogToS", DID.DetailedLogToS);
        SetOne ("Logging.Disk", DID.DiskLogS);
        SetOne ("Logging.Screen", DID.ScreenLogS);
        SetOne ("Logging.Pipe", DID.PipeLogS);
        SetOne ("Logging.Syslog", DID.SysLogS);
        SetOne ("Logging.TransLogNameS", DID.LogFileNameS);
        SetOne ("Logging.SyslogHost", DID.SyslogHostLabel);
        SetOne ("Logging.LogSITEMNGR", DID.LogSITEMNGR);

        (* Update the spin button text. *)

        StrToBuffer (lang, "Logging.tlog0", LogDetailText[0]);
        StrToBuffer (lang, "Logging.tlog1", LogDetailText[1]);
        StrToBuffer (lang, "Logging.tlog2", LogDetailText[2]);
        StrToBuffer (lang, "Logging.tlog3", LogDetailText[3]);
        OS2.WinSendDlgItemMsg (pagehandle, DID.LogDetail, OS2.SPBM_SETARRAY,
                        ADR(LogDetailPtr), OS2.MPFROMSHORT(4));

    END SetLanguage;

(************************************************************************)

PROCEDURE EnableFieldsPass2 (hwnd: OS2.HWND);

    (* Enables or disables some of the dialogue fields. *)

    (********************************************************************)

    PROCEDURE DoOne (field: CARDINAL;  cond: BOOLEAN);

        VAR w: OS2.HWND;

        BEGIN
            w := OS2.WinWindowFromID (hwnd, field);

            (* I have never discovered why WinEnableWindow won't work   *)
            (* if the second parameter is a variable.                   *)

            IF cond THEN
                OS2.WinEnableWindow (w, TRUE);
            ELSE
                OS2.WinEnableWindow (w, FALSE);
            END (*IF*);
        END DoOne;

    (********************************************************************)

    BEGIN
        DoOne (DID.TransferLogBox, FTP);
        DoOne (DID.LogDetail, FTP);
        DoOne (DID.UserLog, FTP);
        DoOne (DID.UserLogName, FTP);
        DoOne (DID.CommonLog, FTP);
        DoOne (DID.CommonLogName, FTP);
        DoOne (DID.LogSITEMNGR, FTP);

        DoOne (DID.TransactionLogBox, FTP);
        DoOne (DID.DetailedLogTo, FTP);
        DoOne (DID.DiskLog, FTP);
        DoOne (DID.ScreenLog, FTP);
        DoOne (DID.PipeLog, FTP);
        DoOne (DID.SysLog, FTP);
        DoOne (DID.LogFileName, FTP);
        DoOne (DID.TransLogName, FTP);

        DoOne (DID.TransactionLogBoxS, SFTP);
        DoOne (DID.DetailedLogToS, SFTP);
        DoOne (DID.DiskLogS, SFTP);
        DoOne (DID.ScreenLogS, SFTP);
        DoOne (DID.PipeLogS, SFTP);
        DoOne (DID.SysLogS, SFTP);
        DoOne (DID.LogFileNameS, SFTP);
        DoOne (DID.TransLogNameS, SFTP);

    END EnableFieldsPass2;

(************************************************************************)
(*                   WHICH SERVERS ARE WE CONFIGURING?                  *)
(************************************************************************)

PROCEDURE SetServer (configFTP, configSFTP: BOOLEAN);

    (* Specifies which server(s) we are configuring. *)

    BEGIN
        FTP := configFTP;
        SFTP := configSFTP;
    END SetServer;

(************************************************************************)
(*                    RESPONDING TO CHECKBOX VALUES                     *)
(************************************************************************)

PROCEDURE QueryButton (hwnd: OS2.HWND;  B: CARDINAL): CARDINAL;

    BEGIN
        RETURN OS2.LONGFROMMR (OS2.WinSendDlgItemMsg (hwnd, B,
                                              OS2.BM_QUERYCHECK, NIL, NIL));
    END QueryButton;

(**************************************************************************)

PROCEDURE EnableLogFields (hwnd: OS2.HWND);

    (* Enables or disables the filename windows, depending on the       *)
    (* checkbox values.                                                 *)

    BEGIN
        (* Transfer logging. *)

        IF QueryButton (hwnd, DID.UserLog) > 0 THEN
            OS2.WinEnableWindow (OS2.WinWindowFromID(hwnd, DID.UserLogName), TRUE);
        ELSE
            OS2.WinEnableWindow (OS2.WinWindowFromID(hwnd, DID.UserLogName), FALSE);
        END (*IF*);
        IF QueryButton (hwnd, DID.CommonLog) > 0 THEN
            OS2.WinEnableWindow (OS2.WinWindowFromID(hwnd, DID.CommonLogName), TRUE);
        ELSE
            OS2.WinEnableWindow (OS2.WinWindowFromID(hwnd, DID.CommonLogName), FALSE);
        END (*IF*);

        (* FTPD settings. *)

        IF QueryButton (hwnd, DID.DiskLog) > 0 THEN
            OS2.WinEnableWindow (OS2.WinWindowFromID(hwnd, DID.LogFileName), TRUE);
            OS2.WinEnableWindow (OS2.WinWindowFromID(hwnd, DID.TransLogName), TRUE);
        ELSE
            OS2.WinEnableWindow (OS2.WinWindowFromID(hwnd, DID.LogFileName), FALSE);
            OS2.WinEnableWindow (OS2.WinWindowFromID(hwnd, DID.TransLogName), FALSE);
        END (*IF*);

        (* SFTPD settings. *)

        IF QueryButton (hwnd, DID.DiskLogS) > 0 THEN
            OS2.WinEnableWindow (OS2.WinWindowFromID(hwnd, DID.LogFileNameS), TRUE);
            OS2.WinEnableWindow (OS2.WinWindowFromID(hwnd, DID.TransLogNameS), TRUE);
        ELSE
            OS2.WinEnableWindow (OS2.WinWindowFromID(hwnd, DID.LogFileNameS), FALSE);
            OS2.WinEnableWindow (OS2.WinWindowFromID(hwnd, DID.TransLogNameS), FALSE);
        END (*IF*);

        (* Syslog. *)

        IF (QueryButton (hwnd, DID.SysLog) > 0) OR (QueryButton (hwnd, DID.SysLogS) > 0) THEN
            OS2.WinEnableWindow (OS2.WinWindowFromID(hwnd, DID.SyslogHostLabel), TRUE);
            OS2.WinEnableWindow (OS2.WinWindowFromID(hwnd, DID.SyslogHost), TRUE);
        ELSE
            OS2.WinEnableWindow (OS2.WinWindowFromID(hwnd, DID.SyslogHostLabel), FALSE);
            OS2.WinEnableWindow (OS2.WinWindowFromID(hwnd, DID.SyslogHost), FALSE);
        END (*IF*);

        (* Now possibly disable some fields depending on FTP/SFTP setting. *)

        EnableFieldsPass2 (hwnd);

    END EnableLogFields;

(************************************************************************)
(*                     LOADING DATA FROM THE INI FILE                   *)
(************************************************************************)

PROCEDURE LoadValues (hwnd: OS2.HWND);

    (* Fills the dialogue elements on the log page with data from the   *)
    (* INI file, or loads default values if they're not in the INI file.*)

    VAR val0, temp: CARDINAL;
        fhini: FHINI;
        name: FilenameString;

    BEGIN
        fhini := OpenINIFiles();

        (* Transfer log level. *)

        IF INIGetCard (fhini, '$SYS', 'LogLevel', val0) THEN
            OldLogLevel := val0;
        ELSE
            val0 := 1;
        END (*IF*);
        OS2.WinSendDlgItemMsg (hwnd, DID.LogDetail, OS2.SPBM_SETCURRENTVALUE,
                                    OS2.MPFROMSHORT(val0), NIL);

        (* "Enable logging" flags.  The value in the INI file is        *)
        (* effectively an array of eight Booleans:                      *)
        (*    bit 0          transaction log to disk   (obsolete)       *)
        (*    bit 1          transaction log to screen (obsolete)       *)
        (*    bit 2          enable user transfer logging               *)
        (*    bit 3          enable common.log transfer logging         *)
        (*    bit 4          transaction log to disk                    *)
        (*    bit 5          transaction log to screen                  *)
        (*    bit 6          transaction log to pipe                    *)
        (*    bit 7          transaction log to syslog                  *)
        (* That is, TransLevel DIV 16 controls the transaction logging, *)
        (* while TransLevel MOD 16 DIV 4 controls the transfer logging. *)
        (* The redundant bits 0 and 1 are a transition arrangement;     *)
        (* originally bits 0 and 1 controlled transaction logging, so   *)
        (* we have to take the OR of two ways to specify the disk and   *)
        (* screen options.                                              *)

        IF INIGetCard (fhini, '$SYS', 'TransLevel', val0) THEN
            OldTransLevel := val0;
        ELSE
            val0 := 52;
        END (*IF*);
        temp := val0 MOD 4;
        val0 := IOR (val0 DIV 4, 4*temp);
        OS2.WinSendDlgItemMsg (hwnd, DID.UserLog, OS2.BM_SETCHECK,
                                    OS2.MPFROMSHORT(ORD(ODD(val0))), NIL);
        val0 := val0 DIV 2;
        OS2.WinSendDlgItemMsg (hwnd, DID.CommonLog, OS2.BM_SETCHECK,
                                    OS2.MPFROMSHORT(ORD(ODD(val0))), NIL);
        val0 := val0 DIV 2;
        OS2.WinSendDlgItemMsg (hwnd, DID.DiskLog, OS2.BM_SETCHECK,
                                    OS2.MPFROMSHORT(ORD(ODD(val0))), NIL);
        val0 := val0 DIV 2;
        OS2.WinSendDlgItemMsg (hwnd, DID.ScreenLog, OS2.BM_SETCHECK,
                                    OS2.MPFROMSHORT(ORD(ODD(val0))), NIL);
        val0 := val0 DIV 2;
        OS2.WinSendDlgItemMsg (hwnd, DID.PipeLog, OS2.BM_SETCHECK,
                                    OS2.MPFROMSHORT(ORD(ODD(val0))), NIL);
        val0 := val0 DIV 2;
        OS2.WinSendDlgItemMsg (hwnd, DID.SysLog, OS2.BM_SETCHECK,
                                    OS2.MPFROMSHORT(ORD(ODD(val0))), NIL);

        (* Similarly for the SFTP logging flags, except that we ignore  *)
        (* the obsolete flags.                                          *)
        (*    bit 4          transaction log to disk                    *)
        (*    bit 5          transaction log to screen                  *)
        (*    bit 6          transaction log to pipe                    *)
        (*    bit 7          transaction log to syslog                  *)
        (* That is, TransLevel DIV 16 controls the transaction logging, *)

        IF INIGetCard (fhini, '$SYS', 'TransLevelS', val0) THEN
            OldTransLevelS := val0;
        ELSE
            val0 := 48;
        END (*IF*);
        val0 := val0 DIV 16;
        OS2.WinSendDlgItemMsg (hwnd, DID.DiskLogS, OS2.BM_SETCHECK,
                                    OS2.MPFROMSHORT(ORD(ODD(val0))), NIL);
        val0 := val0 DIV 2;
        OS2.WinSendDlgItemMsg (hwnd, DID.ScreenLogS, OS2.BM_SETCHECK,
                                    OS2.MPFROMSHORT(ORD(ODD(val0))), NIL);
        val0 := val0 DIV 2;
        OS2.WinSendDlgItemMsg (hwnd, DID.PipeLogS, OS2.BM_SETCHECK,
                                    OS2.MPFROMSHORT(ORD(ODD(val0))), NIL);
        val0 := val0 DIV 2;
        OS2.WinSendDlgItemMsg (hwnd, DID.SysLogS, OS2.BM_SETCHECK,
                                    OS2.MPFROMSHORT(ORD(ODD(val0))), NIL);

        (* Log SITE MNGR commands. *)

        IF NOT INIFetch (fhini, '$SYS', 'LogSITEMNGR', OldLogSITEMNGR) THEN
            OldLogSITEMNGR := TRUE;
        END (*IF*);
        OS2.WinSendDlgItemMsg (hwnd, DID.LogSITEMNGR, OS2.BM_SETCHECK,
                                    OS2.MPFROMSHORT(ORD(OldLogSITEMNGR)), NIL);

        (* Log file names. *)

        IF NOT INIGetString (fhini, '$SYS', 'TransLogName', OldTransLogName) THEN
            OldTransLogName := 'FTPTRANS.LOG';
        END (*IF*);
        OS2.WinSetDlgItemText (hwnd, DID.TransLogName, OldTransLogName);
        IF NOT INIGetString (fhini, '$SYS', 'TransLogNameS', OldTransLogNameS) THEN
            OldTransLogNameS := 'SFTPTRANS.LOG';
        END (*IF*);
        OS2.WinSetDlgItemText (hwnd, DID.TransLogNameS, OldTransLogNameS);

        IF NOT INIGetString (fhini, '$SYS', 'CommonLogName', OldCommonLogName) THEN
            OldCommonLogName := 'COMMON.LOG';
        END (*IF*);
        OS2.WinSetDlgItemText (hwnd, DID.CommonLogName, OldCommonLogName);

        IF NOT INIGetString (fhini, '$SYS', 'UserLogName', OldUserLogName) THEN
            OldUserLogName := 'FTPUSERS.LOG';
        END (*IF*);
        OS2.WinSetDlgItemText (hwnd, DID.UserLogName, OldUserLogName);

        IF INIGetString (fhini, '$SYS', 'SyslogHost', name) THEN
            OldSyslogHost := name;
        ELSE
            OldSyslogHost := '';
            name := '127.0.0.1';
        END (*IF*);
        OS2.WinSetDlgItemText (hwnd, DID.SyslogHost, name);

        CloseINIFiles (fhini);
        EnableLogFields (hwnd);

    END LoadValues;

(************************************************************************)
(*                      STORING DATA TO THE INI FILE                    *)
(************************************************************************)

PROCEDURE StoreData (hwnd: OS2.HWND);

    (* Stores the values on the log page back into the INI file. *)

    VAR value: CARDINAL;
        fhini: FHINI;
        filename: FilenameString;
        bool: BOOLEAN;

    BEGIN
        fhini := OpenINIFiles();

        (* Transfer log level. *)

        value := 0;
        OS2.WinSendDlgItemMsg (hwnd, DID.LogDetail, OS2.SPBM_QUERYVALUE,
                        ADR(value), OS2.MPFROM2SHORT(0,0));
        IF value <> OldLogLevel THEN
            INIPut (fhini, '$SYS', 'LogLevel', value);
        END (*IF*);

        (* "Enable logging" flags.  The value in the INI file is *)
        (* effectively an array of six Booleans.                 *)

        value := 4*( QueryButton (hwnd, DID.UserLog)
                   + 2 * QueryButton (hwnd, DID.CommonLog)
                   + 4 * QueryButton (hwnd, DID.DiskLog)
                   + 8 * QueryButton (hwnd, DID.ScreenLog)
                   + 16 * QueryButton (hwnd, DID.PipeLog)
                   + 32 * QueryButton (hwnd, DID.SysLog) );
        IF value <> OldTransLevel THEN
            INIPut (fhini, '$SYS', 'TransLevel', value);
        END (*IF*);

        (* Now the same for SFTP logging.                 *)

        value := 4*(
                     4 * QueryButton (hwnd, DID.DiskLogS)
                   + 8 * QueryButton (hwnd, DID.ScreenLogS)
                   + 16 * QueryButton (hwnd, DID.PipeLogS)
                   + 32 * QueryButton (hwnd, DID.SysLogS) );
        IF value <> OldTransLevelS THEN
            INIPut (fhini, '$SYS', 'TransLevelS', value);
        END (*IF*);

        (* Log SITE MNGR commands. *)

        bool := QueryButton (hwnd, DID.LogSITEMNGR) > 0;
        IF bool <> OldLogSITEMNGR THEN
            INIPut (fhini, '$SYS', 'LogSITEMNGR', bool);
        END (*IF*);

        (* Log file names. *)

        OS2.WinQueryDlgItemText (hwnd, DID.TransLogName, SIZE(filename),
                                                                 filename);
        IF NOT Strings.Equal (filename, OldTransLogName) THEN
            INIPutString (fhini, '$SYS', 'TransLogName', filename);
        END (*IF*);

        OS2.WinQueryDlgItemText (hwnd, DID.TransLogNameS, SIZE(filename),
                                                                 filename);
        IF NOT Strings.Equal (filename, OldTransLogNameS) THEN
            INIPutString (fhini, '$SYS', 'TransLogNameS', filename);
        END (*IF*);

        OS2.WinQueryDlgItemText (hwnd, DID.CommonLogName, SIZE(filename),
                                                                 filename);
        IF NOT Strings.Equal (filename, OldCommonLogName) THEN
            INIPutString (fhini, '$SYS', 'CommonLogName', filename);
        END (*IF*);
        OS2.WinQueryDlgItemText (hwnd, DID.UserLogName, SIZE(filename),
                                                                 filename);
        IF NOT Strings.Equal (filename, OldUserLogName) THEN
            INIPutString (fhini, '$SYS', 'UserLogName', filename);
        END (*IF*);

        OS2.WinQueryDlgItemText (hwnd, DID.SyslogHost, SIZE(FilenameString),
                                                                 filename);
        IF NOT Strings.Equal (filename, OldSyslogHost) THEN
            INIPutString (fhini, '$SYS', 'SyslogHost', filename);
        END (*IF*);

        CloseINIFiles (fhini);

    END StoreData;

(**************************************************************************)

PROCEDURE ["SysCall"] DialogueProc (hwnd: OS2.HWND;  msg: OS2.ULONG;
                                      mp1, mp2: OS2.MPARAM): OS2.MRESULT;

    VAR NotificationCode, code: CARDINAL;
        dummy: ARRAY [0..15] OF CHAR;

    BEGIN
        IF msg = OS2.WM_INITDLG THEN

            OS2.WinSetWindowPos (hwnd, 0, 0, 0, 0, 0, OS2.SWP_MOVE);
            OS2.WinSendDlgItemMsg (hwnd, DID.LogDetail, OS2.SPBM_SETARRAY,
                        ADR(LogDetailPtr), OS2.MPFROMSHORT(4));
            CommonSettings.WhichServers (FTP, SFTP, dummy);
            LoadValues (hwnd);
            RETURN NIL;

        ELSIF msg = OS2.WM_CONTROL THEN

            NotificationCode := OS2.ULONGFROMMP(mp1) DIV 65536;
            IF NotificationCode = OS2.BN_CLICKED THEN
                EnableLogFields (hwnd);
                RETURN NIL;
            END (*IF*);

        ELSIF msg = OS2.WM_CHAR THEN

            (* Explicitly ignore Esc key, because the default processing  *)
            (* was causing problems.                                      *)

            code := OS2.ULONGFROMMP(mp1);
            IF (IAND (code, OS2.KC_VIRTUALKEY) <> 0) THEN
                code := (OS2.ULONGFROMMP(mp2) DIV 65536) MOD 256;
                IF code = OS2.VK_ESC THEN
                    (* Say we've already handled it. *)
                    RETURN NIL;
                ELSE
                    RETURN OS2.WinDefDlgProc(hwnd, msg, mp1, mp2);
                END (*IF*);
            ELSE
                RETURN OS2.WinDefDlgProc(hwnd, msg, mp1, mp2);
            END (*IF*);

        ELSIF msg = OS2.WM_PRESPARAMCHANGED THEN

            IF ChangeInProgress THEN
                RETURN OS2.WinDefDlgProc(hwnd, msg, mp1, mp2);
            ELSE
                ChangeInProgress := TRUE;
                CommonSettings.UpdateFontFrom (hwnd);
                ChangeInProgress := FALSE;
                RETURN NIL;
            END (*IF*);

        END (*IF*);

        RETURN OS2.WinDefDlgProc (hwnd, msg, mp1, mp2);

    END DialogueProc;

(**************************************************************************)

PROCEDURE CreatePage (notebook: OS2.HWND;  VAR (*OUT*) pgid: CARDINAL): OS2.HWND;

    (* Creates the log page and adds it to the notebook. *)

    VAR Label: ARRAY [0..31] OF CHAR;

    BEGIN
        notebookhandle := notebook;
        pagehandle := OS2.WinLoadDlg(notebook, notebook,
                       DialogueProc,    (* dialogue procedure *)
                       0,                   (* use resources in EXE *)
                       DID.LogPage,                (* dialogue ID *)
                       NIL);                 (* creation parameters *)
        PageID := OS2.ULONGFROMMR (OS2.WinSendMsg (notebook, OS2.BKM_INSERTPAGE,
                         NIL, OS2.MPFROM2SHORT (OS2.BKA_MAJOR+OS2.BKA_AUTOPAGESIZE, OS2.BKA_LAST)));
        pgid := PageID;
        Label := "~Logging";
        OS2.WinSendMsg (notebook, OS2.BKM_SETTABTEXT,
                        CAST(ADDRESS,PageID), ADR(Label));
        OS2.WinSendMsg (notebook, OS2.BKM_SETPAGEWINDOWHWND,
                        CAST(ADDRESS,PageID), CAST(ADDRESS,pagehandle));
        RETURN pagehandle;
    END CreatePage;

(**************************************************************************)

PROCEDURE SetFont (VAR (*IN*) name: CommonSettings.FontName);

    (* Sets the font of the text on this page. *)

    CONST bufsize = CommonSettings.FontNameSize;

    BEGIN
        OS2.WinSetPresParam (pagehandle, OS2.PP_FONTNAMESIZE, bufsize, name);
    END SetFont;

(**************************************************************************)

VAR j: CARDINAL;

BEGIN
    OldLogLevel := 0;
    OldTransLevel := 0;
    OldLogSITEMNGR := FALSE;
    LogDetailText[0] := "No transfer logging";
    LogDetailText[1] := "Log successful transfers";
    LogDetailText[2] := "Log all transfers";
    LogDetailText[3] := "Log all clients";
    FOR j := 0 TO 3 DO
        LogDetailPtr[j] := ADR(LogDetailText[j]);
    END (*FOR*);
    ChangeInProgress := FALSE;
END LogPage.

